@@ -125,6 +125,21 @@ let of_channel ?separator ?excel_tricks fh =
        method close_in() = Pervasives.close_in fh
      end)
 
+let of_string ?separator ?excel_tricks str =
+  of_in_obj ?separator ?excel_tricks
+    (object
+       val mutable position = 0
+       method input buf ofs len =
+         if position >= String.length str
+         then raise End_of_file
+         else
+           ( let actual = min len (String.length str - position) in
+               String.blit str position buf ofs actual ;
+               position <- position + actual ;
+               actual )
+       method close_in() = ()
+     end)
+
 
 (* [fill_in_buf chan] refills in_buf if needed (when empty).  After
    this [in0 < in1] or [in0 = in1 = 0], the latter indicating that
@@ -689,6 +704,28 @@ let rec concat = function
         fun (left_row, right_row) -> List.append left_row right_row
       ) (List.combine left_csv right_csv)
 
+let transpose =
+  (* Suppose the CSV data is presented with the last row first.  Then
+     new rows may be constructed in a tail rec way.  We use mutable
+     rows in order to preserve tail recursiveness. *)
+  (* Return the new 1st row; whether all rows are empty. *)
+  let rec row_of_1st_col tr_row empty = function
+    | [] -> (tr_row, empty)     (* No more rows *)
+    | r :: rows ->
+       match !r with
+       | [] ->                           (* Last row empty *)
+          let tr_row = if tr_row = [] then tr_row else "" :: tr_row in
+          row_of_1st_col tr_row empty rows
+       | a :: tl ->
+          r := tl;
+          let tr_row = if a = "" && tr_row = [] then [] else a :: tr_row in
+          row_of_1st_col tr_row false rows  in
+  let rec tr tr_csv csv =
+    let row, empty = row_of_1st_col [] true csv in (* remove [csv] 1st col *)
+    if empty then List.rev tr_csv
+    else tr (row :: tr_csv) csv in
+  fun csv -> tr [] (List.rev_map ref csv)
+
 let to_array csv =
   Array.of_list (List.map Array.of_list csv)
 
